﻿Public Class G2J
    'Structures declarations
    Structure JprStructure      'Necessary JPR data (in the context of this application)
        Public nm As String         'Name of the Chart
        Public sc As Double         'Scale of the Chart
        Public sr As Double         'Resolution of the Chart
        Public it As String         'Type of image of the Chart
        Public sd As Double         'Size of pixel in meters (no a formal JPF variable)
        Public dm As String         'Geographic datum of the Chart
        Public st As Double         'Datum shift (latitude)
        Public sn As Double         'Datum shift (longitude)
        Public pr As String         'Projection of the Chart
        Public zn As String         'UTM zone of the Chart
        Public pp As String         'Central meridian of the Chart
        Public p1 As String         'Latitude of the origin of the Chart
        Public p2 As String         'Scalefactor of the Chart
        Public p3 As String         'False Easting of the Chart
        Public p4 As String         'Fales Northing of the Chart
        Public p5 As String         'First standard parallel of the Chart
        Public p6 As String         'Second standard parallel of the Chart
    End Structure

    Structure Coordinate        'Referenced Pixel and Geographical coordinates
        Public x As String          'Pixel X
        Public y As String          'Pixel Y
        Public lat As String        'Latitude
        Public lon As String        'Longitude
        Public east As String       'Easting
        Public north As String      'Northing
        Public Valid As Integer     'Application internal variable used for validation
    End Structure

    Structure GdalStructure             'Gdal data about Resolution
        Public ResolutionX As Single        'Horizontal resolotion 
        Public ResolutionY As Single        'Vertical resolution
        Public ResolutionUnit As Integer    'Unit used for resolution (2=DPI/3=Dot per cm)
        Public PixelSizeX As Double         'Horizontal size of a pixel in reality (in meter)
        Public PixelSizeY As Double         'Vertical size of a pixel in reality (in meter)
    End Structure

    'Variable declarations
    Public IniFilePath As String                    'Filename and path to 'g2j.ini'.
    Public IniContent As String                     'Content of 'g2j.ini'.
    Public BatFilePath As String                    'Filename and path to 'OSGeo4W.bat'.
    Public ChartFilePath As String                  'Filename and path to TIFF- or PDF-file.
    Public ChartFile As String                      'Filename TIFF- or PDF-file.
    Public ChartPath As String                      'Path to TIFF- or PDF-file.
    Public ChartName As String                      'Filename TIFF- or PDF-file without extention.
    Public ChartExt As String                       'Extention of TIFF- or PDF-file.
    Public ImageFilePath As String                  'Filename and path to extracted image file (in case PDF)
    Public JprFilePath As String                    'Filename and path to to JPR-file.
    Public GdalContent As String                    'Geo-Information retrieved with GdalInfo
    Public TempFilePath As String                   'Filename and path to a temporary file with the Gdal information
    Public JprData As JprStructure                  'Variable representing the JPR data
    Public ReferencePoint(5) As Coordinate          'Calibration points (topleft, topright, bottomright, bottomleft, center)
    Public VertexPoint(4) As Coordinate             'Vertexpoints (topleft, topright, bottomright, bottomleft)
    Public VertexAmount As Integer = 4              'Amount of vertexpoints
    Public Ratio As Double                          'Ratio between grid coordinate units and pixels
    Public NeatLine As String                       'Variable representing the NEATLINE
    Public NeatLineUsed As Boolean = False          'Indication the NEATLINE was used to construct a vertex polygon
    Public ImageWidth As Integer                    'Width of the image im pixel
    Public ImageHeight As Integer                   'Height of the image in pixel
    Public ImageDiagonal As Double                  'The length of image diagonal
    Public GdalData As GdalStructure                'Variable representing the neccesary Gdal data
    Public ResolutionScaleCode As Integer           'This ariable is a 'returncode' when handling Resoution and Scale

    Public EndLine As String = Chr(13) + Chr(10)    'Variable representing the end a line of text.

    'Subroutine G2J_Load
    'This sub is will be executed starting the application
    '- It enables and sets the Tooltips for buttons and boxes.
    '- It enables/disables buttons
    '- It calls sub 'CheckGdalPath' (to load 'BatFilePath').
    Private Sub G2J_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        'Declarations
        Dim TipText As New ToolTip()

        'Setting up the delays for the ToolTip (TipText).
        TipText.AutoPopDelay = 5000
        TipText.InitialDelay = 1000
        TipText.ReshowDelay = 500

        'Forcing the ToolTip text to be displayed whether or not the form is active.
        TipText.ShowAlways = True

        TipText.SetToolTip(Me.OSButton, "Find or change the batch file to start the Gdal environment (OSGeo4W.bat)")
        TipText.SetToolTip(Me.BatFileValue, "The location of the batchfile to start the Gdal environment (OSGeo4W.bat)")
        TipText.SetToolTip(Me.FindButton, "Find or change the PDF/TIF-file and load the content")
        TipText.SetToolTip(Me.ChartFileValue, "The location of the the PDF/TIF-file")
        TipText.SetToolTip(Me.GdalInfoBox, "The geographic and image parameters of the the PDF/TIF-file, extracted with Gdalinfo")
        TipText.SetToolTip(Me.ExtractButton, "Extract the JPR data from the data profided by Gdalinfo")
        TipText.SetToolTip(Me.JprDataBox, "The extracted JPR data from the data profided by Gdalinfo")
        TipText.SetToolTip(Me.BuildButton, "Create the content of the JPR-file")
        TipText.SetToolTip(Me.SaveJprFileButton, "Save the content of the JPR-file (in the same directory as the PDF/TIF-file)")
        TipText.SetToolTip(Me.JprFileBox, "The  content of the JPR-file")
        TipText.SetToolTip(Me.DisclaimButton, "Show the disclaimer")
        TipText.SetToolTip(Me.EndButton, "End en close the application")

        'Loading DpiBox
        DpiBox.Items.Add("72")
        DpiBox.Items.Add("96")
        DpiBox.Items.Add("100")
        DpiBox.Items.Add("127")
        DpiBox.Items.Add("150")
        DpiBox.Items.Add("200")
        DpiBox.Items.Add("254")
        DpiBox.Items.Add("300")
        DpiBox.Items.Add("508")

        'Initial setting user interface (buttons)
        FindButton.Enabled = False
        ExtractButton.Enabled = False
        BuildButton.Enabled = False
        SaveJprFileButton.Enabled = False
        DpiBox.Enabled = False

        'Initiating the check applications variables
        Call CheckGdalPath()

    End Sub

    'Subroutine CheckGdalPath 
    '- 'CheckGdalPath' will first look for the 'g2j.ini' file in documents directory of user.
    '- If so, it will check the content of m4mm.ini'. the value of 'BatFilePath' must be a exiting file 'OSGeo4W.bat'.
    '  OSGeo4W.bat starts the OSGeo4W shell. This shell is needed to exceute gdalinfo.exe. 
    '  If found this information Is showed In the application window('BatFileValue'). and the button 'FindButton' will be enabled. 
    '- If ini-file is missing or the content of BatFilePath is incorrect, the user will be prompted to took
    '  For the location of 'OSGeo4W.bat' (using the sub 'FindChangeOS').
    Private Sub CheckGdalPath()
        'Declarations
        Dim Answer As Integer
        Dim TempText As String
        'Looking for m4mm.ini
        ' (My.Computer.FileSystem.SpecialDirectories.MyDocuments points to users 'My documents' directory
        IniFilePath = My.Computer.FileSystem.SpecialDirectories.MyDocuments & "\m4mm.ini"
        If My.Computer.FileSystem.FileExists(IniFilePath) = True Then
            IniContent = My.Computer.FileSystem.ReadAllText(IniFilePath)
            'ini-data is ony one line; text will be split in two parts (part(0) is "BatFilePath" and the part(1) is the value for GdalFilePath)
            'Before spliting there will be a check if the separating chr (=) exist
            Answer = InStr(IniContent, "=", CompareMethod.Binary)
            If Answer > 0 Then
                Dim Part() As String = Split(IniContent, "=",,)
                If My.Computer.FileSystem.FileExists(Part(1)) = True Then
                    BatFilePath = Part(1)
                    BatFileValue.Text = BatFilePath
                    'Enabling next step of application OSGeo4W.bat
                    FindButton.Enabled = True
                    FindButton.Select()
                    Exit Sub
                End If
            End If
        End If

        'Initiating the procedure to find 'OSGeo4W.bat'. This batch file is neccesary to open the OSGeo4W enviroment
        TempText = "OSGeo4W.bat wasn't found. Without this file/information the application will not work correctly!"
        MsgBox(TempText, 48, "Warning")
        Call FindChangeOS()

    End Sub

    'Subroutine FindChangeOS
    '- This sub launches the OpenFileDialog to find the file 'OSGeo4W.bat'.
    '- If found this information Is showed In the application window('BatFileValue'). and the button 'FindButton' will be enabled. 
    '- If not found there will be a warning. No chances will be made ('BatFileValue' and 'FindButton').
    Private Sub FindChangeOS()
        'Declarations
        Dim TempFileName As String
        Dim TempText As String
        Dim FindBatFile As New OpenFileDialog

        'Setting variables for Filedialog
        FindBatFile.InitialDirectory = "c:\"
        FindBatFile.Filter = "OSGeo4W-file | OSGeo4W.bat"
        FindBatFile.Multiselect = False

        'Opening dialog
        FindBatFile.ShowDialog()
        TempFileName = FindBatFile.FileName

        'Stopping subroutine if OSGeo4W.bat isn't selected (len TempfileName = 0)
        If Len(TempFileName) = 0 Then
            TempText = "No OSGeo4W.bat was selected. Without this file/information the application will not work correctly!"
            MsgBox(TempText, 48, "Warning")
            Exit Sub
        End If

        'Setting variables
        BatFilePath = TempFileName
        BatFileValue.Text = TempFileName

        'Deleting existing m4mm.ini if exists
        If My.Computer.FileSystem.FileExists(IniFilePath) Then My.Computer.FileSystem.DeleteFile(IniFilePath)

        'Creating text for new m4mm.ini
        TempText = "BatFilePath=" & BatFilePath
        My.Computer.FileSystem.WriteAllText(IniFilePath, TempText, False)

        'Enabling next step of application OSGeo4W.bat
        FindButton.Enabled = True
        FindButton.Select()

    End Sub

    'Function CvtLatLonStr converts the string expression, representing a Latitude or longitude, 
    '(in degree, minutes, secondes and derection) in a string expression, representing a Latitude 
    'or longitude in decimal degree.
    Private Function CvtLatLonStr(LatLonStr As String) As String
        'Declarations
        Dim ValDeg, ValMin, ValSec, ValSign As Double
        Dim TempText1, TempText2 As String
        Dim ChrPointer As Integer

        'Extracting value degrees (ValDeg)
        TempText1 = LatLonStr
        ChrPointer = InStr(TempText1, "d", CompareMethod.Binary)
        TempText2 = Trim(Mid(TempText1, 1, ChrPointer - 1))
        ValDeg = Val(TempText2)

        'Extracting value minutes (ValMin)
        TempText1 = Mid(TempText1, ChrPointer + 1, Len(TempText1) - ChrPointer)
        ChrPointer = InStr(TempText1, "'", CompareMethod.Binary)
        TempText2 = Trim(Mid(TempText1, 1, ChrPointer - 1))
        ValMin = Val(TempText2)

        'Extracting value Secondes (ValSec)
        TempText1 = Mid(TempText1, ChrPointer + 1, Len(TempText1) - ChrPointer)
        ChrPointer = InStr(TempText1, Chr(34), CompareMethod.Binary)
        TempText2 = Trim(Mid(TempText1, 1, ChrPointer - 1))
        ValSec = Val(TempText2)

        'Extracting value Sign (+/-, ValSgn)
        TempText2 = Mid(TempText1, ChrPointer + 1, Len(TempText1) - ChrPointer)
        Select Case TempText2
            Case "W", "S"
                ValSign = -1
            Case "N", "E"
                ValSign = 1
        End Select

        'Creating return value of function and ending
        Return Format(((ValDeg + (ValMin / 60) + (ValSec / 3600)) * ValSign), "0.000000")

    End Function

    'Function CommaToDot converts, in a string representing as number, the comma (,) 
    'as a seperator character into a dot (.).
    Private Function CommaToDot(CommaString As String) As String
        'Declarations
        Dim TempText As String
        Dim ChrPointer As Integer

        'Moving character by character form 'CommaString' to 'TempText'
        'unless it is a comma (,). In this case the comma will be replaced by a dot (.).
        TempText = ""
        For ChrPointer = 1 To Len(CommaString)
            If Mid(CommaString, ChrPointer, 1) <> "," Then
                TempText = TempText + Mid(CommaString, ChrPointer, 1)
            Else
                TempText = TempText + "."
            End If
        Next

        'Returning string to function and ending
        Return TempText
    End Function

    'Function UnderScoreToSpace converts, in a string, the underscore (_) character into a space ( ).
    Private Function UnderScoreToSpace(UnderScoreString As String) As String
        'Declarations
        Dim TempText As String
        Dim ChrPointer As Integer

        'Moving character by character form 'UnderScoreString' to 'TempText'
        'unless it is a comma (,). In this case the comma will be replaced by a dot (.).
        TempText = ""
        For ChrPointer = 1 To Len(UnderScoreString)
            If Mid(UnderScoreString, ChrPointer, 1) <> "_" Then
                TempText = TempText + Mid(UnderScoreString, ChrPointer, 1)
            Else
                TempText = TempText + " "
            End If
        Next

        'Returning string to function and ending
        Return TempText
    End Function

    'Subroutine ClearData clears all the information of variables used to create a JPR file
    Private Sub ClearData()
        'Declarations
        Dim Pointer As Integer

        '- The JPR data
        JprData.nm = "nodata"
        JprData.sc = -1
        JprData.sr = -1
        JprData.sd = -1
        JprData.it = "nodata"
        JprData.dm = "nodata"
        JprData.st = 0
        JprData.sn = 0
        JprData.pr = "nodata"
        JprData.zn = "nodata"
        JprData.pp = "nodata"
        JprData.p1 = "nodata"
        JprData.p2 = "nodata"
        JprData.p3 = "nodata"
        JprData.p4 = "nodata"
        JprData.p5 = "nodata"
        JprData.p6 = "nodata"

        '-  The necessary Gdal data
        GdalData.ResolutionX = -1
        GdalData.ResolutionY = -1
        GdalData.ResolutionUnit = -1
        GdalData.PixelSizeX = -1
        GdalData.PixelSizeX = -1

        '- The reference points
        For Pointer = 1 To 5
            ReferencePoint(Pointer).x = 0
            ReferencePoint(Pointer).y = 0
            ReferencePoint(Pointer).lat = 0
            ReferencePoint(Pointer).lon = 0
            ReferencePoint(Pointer).east = 0
            ReferencePoint(Pointer).north = 0
        Next

        '- The vertex points (like Neatline)
        For Pointer = 1 To VertexAmount
            VertexPoint(Pointer).x = 0
            VertexPoint(Pointer).y = 0
            VertexPoint(Pointer).lat = 0
            VertexPoint(Pointer).lon = 0
            VertexPoint(Pointer).east = 0
            VertexPoint(Pointer).north = 0
        Next
        ReDim VertexPoint(4)

    End Sub

    'Subroutne DisclaimButton shows the disclaimer foor hzns.nl and gdal.org
    Private Sub DisclaimButton_Click(sender As Object, e As EventArgs) Handles DisclaimButton.Click
        'Declarations
        Dim MsgText As String

        'Disclaimer message
        MsgText = "This application is provided 'As is'. The use of the application is on your own risk. " &
            "Direct or indirect damage by using this application is users responsibility/risk, " &
            "not the application-builders. Redistribution by a third party (commercial of non-commercial) " &
            "is prohibit. Download the application direct from https://hzns.nl." & EndLine & EndLine &
            "This application uses gdalinfo, ad part of Gdal (GDAL/OGR contributors (2019). GDAL/OGR " &
            "Geospatial Data Abstraction software Library. Open Source Geospatial Foundation. (https://gdal.org))."
        MsgBox(MsgText, 48, "Disclaimer")

    End Sub

    'Unloading and closing application
    Private Sub EndButton_Click(sender As Object, e As EventArgs) Handles EndButton.Click
        Me.Close()

    End Sub

    'OSButton_Click
    '- This sub call 'FindChangeOS'.
    '- Enables the oppertunity to change OSGeo4W
    Private Sub OSButton_Click(sender As Object, e As EventArgs) Handles OSButton.Click

        Call FindChangeOS()

    End Sub

    'Subroutine FindButton_Click
    '- Selecting and opening a TIF or PDF file
    '- Creating values for variables: ChartFileName, ChartFile, ChartName, JprFilePath, GdalContent
    '- Retrieving WKT from TIF or PDG file
    '- Presenting MapFileContent on the screen
    '- Loading MapFileContent of MapFile into an array (LineContent)
    Private Sub FindButton_Click(sender As Object, e As EventArgs) Handles FindButton.Click
        'Declarations
        Dim TempFileName As String
        Dim TempText As String
        Dim Pointer As Integer
        Dim FileNameLen As Integer
        Dim FindChartFile As New OpenFileDialog

        'Clearing Textboxes and disabling buttons
        GdalInfoBox.Text = ""
        JprDataBox.Text = ""
        JprFileBox.Text = ""
        DpiBox.Text = ""
        DpiBox.Enabled = False
        ExtractButton.Enabled = False
        BuildButton.Enabled = False
        SaveJprFileButton.Enabled = False

        'Setting variables for Filedialog
        FindChartFile.Filter = "TIFF/PDF-file|*.tif; *.tiff; *.pdf"
        FindChartFile.Multiselect = False

        'Opening dialog
        FindChartFile.ShowDialog()
        TempFileName = FindChartFile.FileName

        'Stopping subroutine if no file is selected (len TempfileName = 0)
        If Len(TempFileName) = 0 Then
            TempText = "No Chart was selected!"
            MsgBox(TempText, 48, "Warning")
            Exit Sub
        End If

        'Setting values for variables
        ChartFilePath = TempFileName
        ChartFileValue.Text = TempFileName

        'Retrieve en set values for variables ChartFile and ChartPath
        FileNameLen = Len(TempFileName)
        For Pointer = FileNameLen To 0 Step -1
            If Mid(TempFileName, Pointer, 1) = "\" Then
                ChartPath = Mid(TempFileName, 1, Pointer)
                ChartFile = Mid(TempFileName, Pointer + 1, FileNameLen - Pointer)
                Exit For
            End If
        Next Pointer

        'Retrieving and setting values for variables ChartName , ChartExt and JprFilePath
        FileNameLen = Len(ChartFile)
        For Pointer = FileNameLen To 0 Step -1
            If Mid(ChartFile, Pointer, 1) = "." Then
                ChartName = Mid(ChartFile, 1, Pointer - 1)
                ChartExt = Mid(ChartFile, Pointer + 1, FileNameLen - Pointer)
                JprFilePath = ChartPath & ChartName & ".jpr"
                Exit For
            End If
        Next Pointer

        'Retrieving Geo-information with GdalInfo
        '- Setting varialbel TempFilePath
        '  (My.Computer.FileSystem.SpecialDirectories.Temp point to users "Temporary" directory)
        TempFilePath = My.Computer.FileSystem.SpecialDirectories.Temp & "\" & "g2j.tmp"
        '-test
        MsgBox(BatFilePath & " gdalinfo.exe " & ChartFilePath & " > " & TempFilePath,, Len(BatFilePath & " gdalinfo.exe " & ChartFilePath & " > " & TempFilePath))
        '- Retrieving Gdal (WKT) data and copying to g2j.tmp
        '  Wait:=True > to be sure process is finalised before executing next statement.
        '  This prevents an error! (trial and error!)
        Call Shell(BatFilePath & " gdalinfo.exe " & ChartFilePath & " > " & TempFilePath, AppWinStyle.Hide, Wait:=True)

        '- Copying content of g2j.tmp to "GdalContent"
        GdalContent = My.Computer.FileSystem.ReadAllText(TempFilePath)

        '- Loading GdalContent into 'GdalInfoBox'
        GdalInfoBox.Text = GdalContent

        '- Deleting tempfile g2j.tmp
        My.Computer.FileSystem.DeleteFile(TempFilePath)

        'Testing for Geographic data (looking for the keyword 'PROJCRS')
        '- If not: Warning
        '- If so: enabling 'ExtractButton' for next step
        Pointer = InStr(GdalContent, "PROJCRS", CompareMethod.Binary)
        If Pointer = 0 Then
            MsgBox("The TIFF/PDF file doesn't contain geographic data!", 48, "Warning")
            'Disabling next steps in the application
            ExtractButton.Enabled = False
            BuildButton.Enabled = False
            SaveJprFileButton.Enabled = False
        Else
            'Enabling next steps in the application
            ExtractButton.Enabled = True
            ExtractButton.Select()
        End If

        'Handling case CharFileExt=pdf
        'Looking for a corresponding imagefile
        ImageFilePath = ""
        If LCase(ChartExt) = "pdf" Then
            TempText = ChartPath & ChartName & ".png"
            If My.Computer.FileSystem.FileExists(TempText) = True Then
                ImageFilePath = TempText
                JprData.it = "png"
            End If
            TempText = ChartPath & ChartName & ".tif"
            If My.Computer.FileSystem.FileExists(TempText) = True Then
                ImageFilePath = TempText
                JprData.it = "tif"
            End If
            TempText = ChartPath & ChartName & ".tiff"
            If My.Computer.FileSystem.FileExists(TempText) = True Then
                ImageFilePath = TempText
                JprData.it = "tif"
            End If
        End If

    End Sub

    'Sub ExtractButton_Click is retrieving/extracting information to create a JPR file
    Private Sub ExtractButton_Click(sender As Object, e As EventArgs) Handles ExtractButton.Click
        'Declarations
        Dim DataLine() As String = Split(GdalContent, EndLine)          'Splitting GdalContent in to lines
        Dim LineCount As Integer = DataLine.Length - 1                  'Determening number of lines
        Dim Counter As Integer
        Dim Pointer(1) As Integer
        Dim TempText As String
        Dim Answer As Integer
        Dim UseImage As Boolean
        Dim MsgText As String
        Dim TempCode As Integer
        Dim ImageData As Image

        'Clearing all data
        Call ClearData()
        JprDataBox.Text = ""
        JprFileBox.Text = ""
        DpiBox.Enabled = False
        DpiBox.Text = ""

        'Asking fort using size fo image file instead of PDF-file (in case PDE and a corresopinding image file)
        UseImage = False
        If ImageFilePath <> "" Then
            Answer = MsgBox("A corresponding image file for the PDF-file is found." & EndLine & "Do you want to use the height and Width this image file?", 36, "Question")
            If Answer = 6 Then UseImage = True '6 = yes
        End If

        'Retrieving Nm (name of map) and graphic type (it)
        JprData.nm = ChartName
        JprDataBox.Text = JprDataBox.Text & "nm (chartname) = " & JprData.nm & EndLine

        If UseImage = False Then
            JprData.it = Mid(ChartFile, Len(ChartName) + 2, 3)
        Else
            TempText = ChartPath & ChartName & ".png"
            If My.Computer.FileSystem.FileExists(TempText) = True Then
                ImageFilePath = TempText
                JprData.it = "png"
            End If
            TempText = ChartPath & ChartName & ".tif"
            If My.Computer.FileSystem.FileExists(TempText) = True Then
                ImageFilePath = TempText
                JprData.it = "tif"
            End If
            TempText = ChartPath & ChartName & ".tiff"
            If My.Computer.FileSystem.FileExists(TempText) = True Then
                ImageFilePath = TempText
                JprData.it = "tif"
            End If

        End If


        JprDataBox.Text = JprDataBox.Text & "it (graphics type) = " & JprData.it & EndLine

        'Retrieving/extracting data with a "more or less" one on one relation
        For Counter = 0 To LineCount

            'Retrieving/extracting dm (geographic datum)
            If InStr(DataLine(Counter), "DATUM", CompareMethod.Binary) > 0 Then
                TempText = Trim(LCase(UnderScoreToSpace(DataLine(Counter))))
                Pointer(0) = InStr(TempText, "[", CompareMethod.Binary)
                TempText = Mid(TempText, Pointer(0) + 2, Len(TempText) - Pointer(0) - 3)
                JprData.dm = TempText
                JprDataBox.Text = JprDataBox.Text & "dm (datum) = " & JprData.dm & EndLine
            End If

            'Retrieving/extracting pr (projection)
            If InStr(DataLine(Counter), "PROJECTION", CompareMethod.Binary) > 0 Then
                Pointer(0) = InStr(DataLine(Counter), "[", CompareMethod.Binary)
                Pointer(1) = InStr(DataLine(Counter), "]", CompareMethod.Binary)
                TempText = UnderScoreToSpace(LCase(Mid(DataLine(Counter), Pointer(0) + 2, Pointer(1) - Pointer(0) - 3)))
                JprData.pr = TempText
                JprDataBox.Text = JprDataBox.Text & "pr (projection) = " & JprData.pr & EndLine
            End If

            'Retrieving/extracting pp (central meridian)
            If InStr(LCase(DataLine(Counter)), "central_meridian", CompareMethod.Binary) > 0 Then
                Pointer(0) = InStr(DataLine(Counter), ",", CompareMethod.Binary)
                Pointer(1) = InStr(DataLine(Counter), "]", CompareMethod.Binary)
                JprData.pp = Trim(Mid(DataLine(Counter), Pointer(0) + 1, Pointer(1) - Pointer(0) - 1))
                JprDataBox.Text = JprDataBox.Text & "pp (central meridian) = " & JprData.pp & EndLine
            End If

            'Retrieving/extracting p1 (latitude of origin)
            If InStr(LCase(DataLine(Counter)), "latitude_of_origin", CompareMethod.Binary) > 0 Then
                Pointer(0) = InStr(DataLine(Counter), ",", CompareMethod.Binary)
                Pointer(1) = InStr(DataLine(Counter), "]", CompareMethod.Binary)
                JprData.p1 = Trim(Mid(DataLine(Counter), Pointer(0) + 1, Pointer(1) - Pointer(0) - 1))
                JprDataBox.Text = JprDataBox.Text & "p1 (latitude of origin) = " & JprData.p1 & EndLine
            End If

            'Retrieving/extracting p2 (scale factor)
            If InStr(LCase(DataLine(Counter)), "scale_factor", CompareMethod.Binary) > 0 Then
                Pointer(0) = InStr(DataLine(Counter), ",", CompareMethod.Binary)
                Pointer(1) = InStr(DataLine(Counter), "]", CompareMethod.Binary)
                JprData.p2 = Trim(Mid(DataLine(Counter), Pointer(0) + 1, Pointer(1) - Pointer(0) - 1))
                JprDataBox.Text = JprDataBox.Text & "p2 (scale factor) = " & JprData.p2 & EndLine
            End If

            'Retrieving/extracting p3 (false northing)
            If InStr(LCase(DataLine(Counter)), "false_northing", CompareMethod.Binary) > 0 Then
                Pointer(0) = InStr(DataLine(Counter), ",", CompareMethod.Binary)
                Pointer(1) = InStr(DataLine(Counter), "]", CompareMethod.Binary)
                JprData.p3 = Trim(Mid(DataLine(Counter), Pointer(0) + 1, Pointer(1) - Pointer(0) - 1))
                JprDataBox.Text = JprDataBox.Text & "p3 (false northing) = " & JprData.p3 & EndLine
            End If

            'Retrieving/extracting p4 (false easting)
            If InStr(LCase(DataLine(Counter)), "false_easting", CompareMethod.Binary) > 0 Then
                Pointer(0) = InStr(DataLine(Counter), ",", CompareMethod.Binary)
                Pointer(1) = InStr(DataLine(Counter), "]", CompareMethod.Binary)
                JprData.p4 = Trim(Mid(DataLine(Counter), Pointer(0) + 1, Pointer(1) - Pointer(0) - 1))
                JprDataBox.Text = JprDataBox.Text & "p4 (false easting) = " & JprData.p4 & EndLine
            End If

            'Retrieving/extracting p5 (standard parallel 1)
            If InStr(LCase(DataLine(Counter)), "standard_parallel_1", CompareMethod.Binary) > 0 Then
                Pointer(0) = InStr(DataLine(Counter), ",", CompareMethod.Binary)
                Pointer(1) = InStr(DataLine(Counter), "]", CompareMethod.Binary)
                JprData.p5 = Trim(Mid(DataLine(Counter), Pointer(0) + 1, Pointer(1) - Pointer(0) - 1))
                JprDataBox.Text = JprDataBox.Text & "p5 (standard parallel 1) = " & JprData.p5 & EndLine
            End If

            'Retrieving/extracting p6 (standard parallel 2)
            If InStr(LCase(DataLine(Counter)), "standard_parallel_2", CompareMethod.Binary) > 0 Then
                Pointer(0) = InStr(DataLine(Counter), ",", CompareMethod.Binary)
                Pointer(1) = InStr(DataLine(Counter), "]", CompareMethod.Binary)
                JprData.p6 = Trim(Mid(DataLine(Counter), Pointer(0) + 1, Pointer(1) - Pointer(0) - 1))
                JprDataBox.Text = JprDataBox.Text & "p6 (standard parallel 2) = " & JprData.p6 & EndLine
            End If

            'Retrieving/extracting NEATLINE
            If InStr(DataLine(Counter), "NEATLINE=POLYGON", CompareMethod.Binary) > 0 Then
                NeatLine = Trim(DataLine(Counter))
            End If

        Next

        'Retrieving/extracting width, Height and center of the image in pixel and their geographical coordinates
        'needed for Reference and vertex points
        For Counter = 0 To LineCount

            '- Retrieving width, Height and center in pixel from GdalData
            If InStr(LCase(DataLine(Counter)), "size is", CompareMethod.Binary) > 0 Then
                If UseImage = True Then
                    '- Retrieving image width en Height of image file (in case PDF and image file found)
                    ImageData = Image.FromFile(ImageFilePath)
                    ImageHeight = ImageData.Height
                    ImageWidth = ImageData.Width
                Else
                    '- Retrieving width, Height and center in pixel from GdalData
                    Dim Size() As String = Split(Mid(DataLine(Counter), 8, Len(DataLine(Counter)) - 7), ",", CompareMethod.Binary)
                    ImageWidth = Val(Size(0))
                    ImageHeight = Val(Size(1))
                End If

                '- Setting Pixel coordinates of the upper left, upper right, lower right, lower left and center
                ReferencePoint(1).x = 0
                ReferencePoint(1).y = 0
                ReferencePoint(2).x = ImageWidth - 1
                ReferencePoint(2).y = 0
                ReferencePoint(3).x = ImageWidth - 1
                ReferencePoint(3).y = ImageHeight - 1
                ReferencePoint(4).x = 0
                ReferencePoint(4).y = ImageHeight - 1
                ReferencePoint(5).x = ImageWidth * 0.5 - 1
                ReferencePoint(5).y = ImageHeight * 0.5 - 1
            End If


            '- Retrieving and setting the geographical coordinate of the upper left corner
            If InStr(DataLine(Counter), "Upper Left", CompareMethod.Binary) > 0 Then
                Dim TempString1() As String = Split(DataLine(Counter), ") (", CompareMethod.Binary)
                Dim TempString2() As String = Split(TempString1(1), ",", CompareMethod.Binary)
                ReferencePoint(1).lon = CvtLatLonStr(TempString2(0))
                ReferencePoint(1).lat = CvtLatLonStr(Mid(TempString2(1), 1, Len(TempString2(1)) - 1))
            End If

            '- Retrieving and setting the geographical coordinate of the upper right corner
            If InStr(DataLine(Counter), "Upper Right", CompareMethod.Binary) > 0 Then
                Dim TempString1() As String = Split(DataLine(Counter), ") (", CompareMethod.Binary)
                Dim TempString2() As String = Split(TempString1(1), ",", CompareMethod.Binary)
                ReferencePoint(2).lon = CvtLatLonStr(TempString2(0))
                ReferencePoint(2).lat = CvtLatLonStr(Mid(TempString2(1), 1, Len(TempString2(1)) - 1))
            End If

            '- Retrieving and setting the geographical coordinate of the lower right corner
            If InStr(DataLine(Counter), "Lower Right", CompareMethod.Binary) > 0 Then
                Dim TempString1() As String = Split(DataLine(Counter), ") (", CompareMethod.Binary)
                Dim TempString2() As String = Split(TempString1(1), ",", CompareMethod.Binary)
                ReferencePoint(3).lon = CvtLatLonStr(TempString2(0))
                ReferencePoint(3).lat = CvtLatLonStr(Mid(TempString2(1), 1, Len(TempString2(1)) - 1))
            End If

            '- Retrieving and setting the geographical coordinate of the lower left corner
            If InStr(DataLine(Counter), "Lower Left", CompareMethod.Binary) > 0 Then
                Dim TempString1() As String = Split(DataLine(Counter), ") (", CompareMethod.Binary)
                Dim TempString2() As String = Split(TempString1(1), ",", CompareMethod.Binary)
                ReferencePoint(4).lon = CvtLatLonStr(TempString2(0))
                ReferencePoint(4).lat = CvtLatLonStr(Mid(TempString2(1), 1, Len(TempString2(1)) - 1))
            End If

            '- Retrieving and setting the geographical coordinate of the center
            If InStr(DataLine(Counter), "Center  ", CompareMethod.Binary) > 0 Then
                Dim TempString1() As String = Split(DataLine(Counter), ") (", CompareMethod.Binary)
                Dim TempString2() As String = Split(TempString1(1), ",", CompareMethod.Binary)
                ReferencePoint(5).lon = CvtLatLonStr(TempString2(0))
                ReferencePoint(5).lat = CvtLatLonStr(Mid(TempString2(1), 1, Len(TempString2(1)) - 1))
            End If

        Next

        '- Copying reference points to 'JprDataBox'
        For Pointer(0) = 1 To 5
            JprDataBox.Text = JprDataBox.Text & "rp" & Pointer(0) & " (Reference point " & Pointer(0) & ") = " &
                CommaToDot(CStr(ReferencePoint(Pointer(0)).lat)) & ", " &
                CommaToDot(CStr(ReferencePoint(Pointer(0)).lon)) & ", " &
                CommaToDot(CStr(ReferencePoint(Pointer(0)).x)) & ", " &
                CommaToDot(CStr(ReferencePoint(Pointer(0)).y)) & EndLine
        Next

        'Asking if the map has a collar.
        '- If not: Vertex polygon will be created based on the reference points (1 to 4) and copied to JprDataBox
        '  (Msgbox appearance: 4 = Yes/No, 32 = Question mark, 256 = second button default, 292 = total)
        Answer = MsgBox("Does the map has a collar?", 292, "Question")
        If Answer = 7 Then
            For Pointer(0) = 1 To VertexAmount
                VertexPoint(Pointer(0)) = ReferencePoint(Pointer(0))
                JprDataBox.Text = JprDataBox.Text & "vp" & Pointer(0) & " (Vertex point " & Pointer(0) & ") = " &
                CommaToDot(CStr(VertexPoint(Pointer(0)).x)) & ", " &
                CommaToDot(CStr(VertexPoint(Pointer(0)).y)) & EndLine
            Next
        Else
            If InStr(GdalContent, "NEATLINE=POLYGON", CompareMethod.Binary) > 0 Then
                NeatLineUsed = False
                '- Constructing the vertex poligon based on the  grid coordinates in the NEATLINE
                Answer = MsgBox("Use the NEATLINE to construct the vertex polygon?", 292, "Question")
                If Answer = 6 Then
                    Ratio = UnitRatio()
                    If Ratio = 0 Then
                        MsgBox("The coordinate grid data are inconsitent, the construction of a vertex polygon isn't possible.", 48, "Warning")
                    Else
                        Call SplitNeatLine()    'Extracting grid coordinates
                        Call CalculateXY()      'Conversion of grid coordinates in to pixel coordinates
                        For Pointer(0) = 1 To VertexPoint.Length - 1
                            'writing to JprboxText
                            JprDataBox.Text = JprDataBox.Text & "vp" & Pointer(0) & " (Vertex point " & Pointer(0) & ") = " &
                                CommaToDot(CStr(VertexPoint(Pointer(0)).x)) & ", " &
                                CommaToDot(CStr(VertexPoint(Pointer(0)).y)) & EndLine
                        Next
                        NeatLineUsed = True
                    End If
                End If
            End If

        End If

        'Handling UTM case (UTM is a special case Transverse Mercator)
        'Retrieving/ extracting zn (UTM zone data) 
        'projcrs filter is used to point to the correct utm zone (in the WKT)
        For Counter = 0 To LineCount
            If InStr(LCase(DataLine(Counter)), "utm zone", CompareMethod.Binary) > 0 And InStr(LCase(DataLine(Counter)), "projcrs", CompareMethod.Binary) Then
                Pointer(0) = InStr(LCase(DataLine(Counter)), "utm zone", CompareMethod.Binary)
                TempText = Mid(DataLine(Counter), Pointer(0) + 9, 3)
                JprData.zn = TempText
                Select Case Mid(JprData.zn, 3, 1)
                    Case "N", "n"
                        JprData.zn = Mid(JprData.zn, 1, 2) & "t"
                    Case "S", "s"
                        JprData.zn = Mid(JprData.zn, 1, 2) & "j"
                    Case Else
                        'if no N or S: the t or j will be based on latitude of center map (reference point 5)
                        If Val(CommaToDot(CStr(ReferencePoint(5).lat))) >= 0 Then
                            JprData.zn = Mid(JprData.zn, 1, 2) & "t"
                        Else
                            JprData.zn = Mid(JprData.zn, 1, 2) & "j"
                        End If
                End Select
                JprDataBox.Text = JprDataBox.Text & "zn (UTM zone) = " & JprData.zn & EndLine
                'Changing Projection tu UTM
                JprData.pr = "utm"
                JprDataBox.Text = JprDataBox.Text & "pr is changed to 'utm'" & EndLine
            End If
        Next

        'Retrieving/extracting data to calculate the scale (sc) and the resolution (sr)
        For Counter = 0 To LineCount
            If InStr(DataLine(Counter), "TIFFTAG_RESOLUTIONUNIT", CompareMethod.Binary) > 0 Then
                Dim TempString1() As String = Split(DataLine(Counter), "=", CompareMethod.Binary)
                GdalData.ResolutionUnit = Val(Mid(TempString1(1), 1, 1))
            End If

            If InStr(DataLine(Counter), "TIFFTAG_XRESOLUTION", CompareMethod.Binary) > 0 Then
                Dim TempString1() As String = Split(DataLine(Counter), "=", CompareMethod.Binary)
                GdalData.ResolutionX = Val(TempString1(1))
            End If

            If InStr(DataLine(Counter), "TIFFTAG_YRESOLUTION", CompareMethod.Binary) > 0 Then
                Dim TempString1() As String = Split(DataLine(Counter), "=", CompareMethod.Binary)
                GdalData.ResolutionY = Val(TempString1(1))
            End If

            If InStr(DataLine(Counter), "Pixel Size =", CompareMethod.Binary) > 0 Then
                Dim TempString1() As String = Split(DataLine(Counter), "(", CompareMethod.Binary)
                Dim TempString2() As String = Split(TempString1(1), ",", CompareMethod.Binary)
                GdalData.PixelSizeX = System.Math.Abs(Val(TempString2(0)))
                GdalData.PixelSizeY = System.Math.Abs(Val(Mid(TempString2(1), 1, Len(TempString2(1)) - 1)))
            End If
        Next

        'Reseting 'ResolutionScaleCode'
        ResolutionScaleCode = 0

        'Handling the resolution (sr)

        'Determining code for msg no resolution could be determined
        If GdalData.ResolutionX <> GdalData.ResolutionY Then ResolutionScaleCode = ResolutionScaleCode + 1
        If GdalData.ResolutionX <= 0 Or GdalData.ResolutionY <= 0 Then ResolutionScaleCode= ResolutionScaleCode + 2
        If GdalData.ResolutionUnit < 2 Or GdalData.ResolutionUnit > 3 Then ResolutionScaleCode = ResolutionScaleCode + 4

        'Retrieving resolution (sr) 
        If GdalData.ResolutionUnit = 3 Then
            JprData.sr = GdalData.ResolutionX * 2.54
            JprDataBox.Text = JprDataBox.Text & "sr (resolution) = " & JprData.sr & EndLine
        End If
        If GdalData.ResolutionUnit = 2 Then
            JprData.sr = GdalData.ResolutionX
            JprDataBox.Text = JprDataBox.Text & "sr (resolution) = " & JprData.sr & EndLine
        End If

        'Handling the scale
        '- If no pixeldata: GdalData.PixelSizeX and GdalData.PixelSizeY has its default -1
        If GdalData.PixelSizeX < 0 Then ResolutionScaleCode = ResolutionScaleCode + 8

        '- The pixelsize X/Y can slidy differ. If less then 0.5% it is OK
        If GdalData.PixelSizeX / GdalData.PixelSizeY > 1.005 Or GdalData.PixelSizeX / GdalData.PixelSizeY < 0.995 Then
            ResolutionScaleCode = ResolutionScaleCode + 16
        End If

        '- Calculating scale (sc) ("if everything when well")
        If ResolutionScaleCode = 0 Then
            JprData.sd = (GdalData.PixelSizeX + GdalData.PixelSizeY) / 2
            JprData.sc = System.Math.Round(JprData.sd * 100 * (JprData.sr / 2.54))
            JprDataBox.Text = JprDataBox.Text & "sc (scale) = " & JprData.sc & EndLine
        End If

        'Creating the 'MsgText' based on ResolutionScaleCode 
        If ResolutionScaleCode > 0 Then
            TempCode = ResolutionScaleCode
            MsgText = "The resolution and/or scale couldn't determined! (" & ResolutionScaleCode & ")" & EndLine
            If TempCode >= 16 Then
                TempCode = TempCode - 16
                MsgText = MsgText & "- The heigth/width of a pixel differs to much" & EndLine
            End If
            If TempCode >= 8 Then
                TempCode = TempCode - 8
                MsgText = MsgText & "- The pixel size couldn't be determined" & EndLine
            End If
            If TempCode >= 4 Then
                TempCode = TempCode - 4
                MsgText = MsgText & "- The resolution unit couldn't be determined" & EndLine
            End If
            If TempCode >= 2 Then
                TempCode = TempCode - 2
                MsgText = MsgText & "- The heigth and/or width resolution couldn't be determined" & EndLine
            End If
            If TempCode = 1 Then
                MsgText = MsgText & "- The heigth and/or width resolution differ" & EndLine
            End If
            JprDataBox.Text = JprDataBox.Text & MsgText
        End If

        'Enabling next step of the application
        BuildButton.Enabled = True
        BuildButton.Select()

    End Sub

    'The subroutine BuildButton_Click Creates the contents of the JPR file. 
    Private Sub BuildButton_Click(sender As Object, e As EventArgs) Handles BuildButton.Click
        'Declarations
        Dim Pointer(2) As Integer

        'Cppying Data to JprFileBox
        '- Standard values
        JprFileBox.Text = "//This file was created Using Gdal data To JPR (G2J) by www.hzns.nl" & EndLine
        JprFileBox.Text = JprFileBox.Text & "nm= " & JprData.nm & EndLine
        JprFileBox.Text = JprFileBox.Text & "it=" & JprData.it & EndLine
        If JprData.it = "pdf" Then
            JprFileBox.Text = JprFileBox.Text & "//The PDF-file has To be converted into a graphics format (256 colors/8 bit)." & EndLine &
                "//Change the value 'it' into the proper format (gif, jpg ,png, bmp  or tif)." & EndLine
        End If

        '- Datum values
        If JprData.dm <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "dm=" & TranslateDatum(JprData.dm) & EndLine
        JprFileBox.Text = JprFileBox.Text & "st=" & JprData.st & EndLine
        JprFileBox.Text = JprFileBox.Text & "sn=" & JprData.sn & EndLine

        '- Projection values
        If JprData.pr <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "pr=" & TranslateProjection(JprData.pr) & EndLine
        Select Case TranslateProjection(JprData.pr)
            Case "mercator"
                If JprData.p1 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p1=" & JprData.p1 & EndLine
            Case "transverse mercator"
                If JprData.pp <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "pp=" & JprData.pp & EndLine
                If JprData.p1 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p1=" & JprData.p1 & EndLine
                If JprData.p2 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p2=" & JprData.p2 & EndLine
                If JprData.p3 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p3=" & JprData.p3 & EndLine
                If JprData.p4 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p4=" & JprData.p4 & EndLine
            Case "utm"
                If JprData.pp <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "pp=" & JprData.pp & EndLine
                If JprData.p1 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p1=" & JprData.p1 & EndLine
                If JprData.p2 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p2=" & JprData.p2 & EndLine
                If JprData.p3 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p3=" & JprData.p3 & EndLine
                If JprData.p4 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p4=" & JprData.p4 & EndLine
                If JprData.zn <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "zn=" & JprData.zn & EndLine
            Case "lambert conformal conic"
                If JprData.pp <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "pp=" & JprData.pp & EndLine
                If JprData.p1 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p1=" & JprData.p1 & EndLine
                If JprData.p5 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p5=" & JprData.p5 & EndLine
                If JprData.p6 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p6=" & JprData.p6 & EndLine
            Case Else
                JprFileBox.Text = JprFileBox.Text & "//Your map program may possibly not supports this projection." & EndLine
                If JprData.pp <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "pp=" & JprData.pp & EndLine
                If JprData.p1 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p1=" & JprData.p1 & EndLine
                If JprData.p2 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p2=" & JprData.p2 & EndLine
                If JprData.p3 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p3=" & JprData.p3 & EndLine
                If JprData.p4 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p4=" & JprData.p4 & EndLine
                If JprData.p5 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p5=" & JprData.p5 & EndLine
                If JprData.p6 <> "nodata" Then JprFileBox.Text = JprFileBox.Text & "p6=" & JprData.p6 & EndLine
        End Select

        '- Resolution and Scale values
        If JprData.sr <> -1 Then JprFileBox.Text = JprFileBox.Text & "sr=" & JprData.sr & EndLine
        If JprData.sc <> -1 Then JprFileBox.Text = JprFileBox.Text & "sc=" & JprData.sc & EndLine

        '- Calibration values
        For Pointer(0) = 1 To 5
            JprFileBox.Text = JprFileBox.Text & "rp" & Pointer(0) & "=" &
                CommaToDot(CStr(ReferencePoint(Pointer(0)).lat)) & "," &
                CommaToDot(CStr(ReferencePoint(Pointer(0)).lon)) & "," &
                CommaToDot(CStr(ReferencePoint(Pointer(0)).x)) & "," &
                CommaToDot(CStr(ReferencePoint(Pointer(0)).y)) & EndLine
        Next

        '- Vertex values
        If VertexPoint(3).x <> 0 Then
            If NeatLineUsed = True Then
                JprFileBox.Text = JprFileBox.Text & "//The vertex polygon was constructed based on a NEATLINE. Please check!" & EndLine
            End If
            For Pointer(0) = 1 To 4
                JprFileBox.Text = JprFileBox.Text & "vp" & Pointer(0) & "=" &
                    CommaToDot(CStr(VertexPoint(Pointer(0)).x)) & "," &
                    CommaToDot(CStr(VertexPoint(Pointer(0)).y)) & EndLine
            Next

        End If

        'Enabling next step of the application
        SaveJprFileButton.Enabled = True
        SaveJprFileButton.Select()

        'Coditional (only if relevant data are aviable) enabling DpiBox for next step of the application
        If ResolutionScaleCode = 0 Then
            DpiBox.Enabled = True
            DpiBox.Text = JprData.sr
        End If

    End Sub

    'The function 'TranslateProjection' translates the Gdal value for the projection
    'to the proper JPR value.
    Private Function TranslateProjection(GdalProjection As String) As String
        'Declarations
        Dim TempProjection As String

        Select Case GdalProjection
            Case "lcc", "lambert cc", "lambert conformal"
                TempProjection = "lambert conformal conic"
            Case "tm"
                TempProjection = "transverse mercator"
            Case "universal transverse mercator"
                TempProjection = "utm"
            Case "merc"
                TempProjection = "mercator"
            Case Else
                TempProjection = GdalProjection
        End Select

        Return TempProjection
    End Function

    'The function TranslateDatum translates the Gdal value for the datum
    'to the proper JPR value. This function gives the flexibility to translate weard projection names into
    'more common names (although the parameters of both datums must be more or less the same).
    Private Function TranslateDatum(GdalDatum As String) As String
        'Declararions
        Dim TempDatum As String

        Select Case GdalDatum
            Case "north american datum 1983"
                TempDatum = "nad83"
            Case "north american datum 1927"
                TempDatum = "nad27"
            Case "wgs 1984", "european terrestrial reference system 1989", "world geodetic system 1984"
                TempDatum = "wgs84"
            Case Else
                TempDatum = GdalDatum
        End Select

        Return TempDatum
    End Function

    'Subroutine SaveJprFileButton_Click saves the JrpFileBox.text to a JPR-file in the same directory
    'as the orginal TIF/PDF file. If this file exists, a warning will be prompted.
    Private Sub SaveJprFileButton_Click(sender As Object, e As EventArgs) Handles SaveJprFileButton.Click
        'Declararions
        Dim Answer As Integer

        'Checking for an existing JPR file
        If My.Computer.FileSystem.FileExists(JprFilePath) = True Then
            '(Msgbox appearance: 4 = Yes/No, 32 = Question mark, 256 = second button default, 292 = total)
            Answer = MsgBox("JPR-file allready exits. Overwrite the existing file?", 292, "Warning")
            If Answer = 6 Then
                'Overwriting existing JPR-file if answer on MsgBox is Yes (Answer = 6)
                My.Computer.FileSystem.WriteAllText(JprFilePath, JprFileBox.Text, False)
            End If
        Else
            'Creating JPR-file
            My.Computer.FileSystem.WriteAllText(JprFilePath, JprFileBox.Text, False)
        End If

    End Sub

    'Subroutine DipBox_LostFocus handles the cases LostFocus and SelectedValueChanged (with focus on DpiBox)
    '- recalculating the map scale
    Private Sub DpiBox_LostFocus(sender As Object, e As EventArgs) Handles DpiBox.LostFocus, DpiBox.SelectedValueChanged
        If JprDataBox.Text <> "" Then
            If JprData.sr <> Val(DpiBox.Text) Then
                JprData.sr = Val(DpiBox.Text)
                JprData.sc = System.Math.Round(JprData.sd * 100 * (JprData.sr / 2.54))
                BuildButton_Click(sender, e)
            End If
        End If

    End Sub

    'Subroutine DipBox_PressKey handles the case "Enter"-key was used (with focus on DpiBox)
    '- recalculaing the map scale
    Private Sub DpiBox_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DpiBox.KeyPress
        If JprDataBox.Text <> "" And e.KeyChar = Chr(13) Then
            If JprData.sr <> Val(DpiBox.Text) Then
                JprData.sr = Val(DpiBox.Text)
                JprData.sc = System.Math.Round(JprData.sd * 100 * (JprData.sr / 2.54))
                BuildButton_Click(sender, e)
            End If
        End If

    End Sub

    'The Function UnitRatio calculates the ratio between the pixelsize and unitsize of 
    'the grid coordinates. This ratio is used to conctruct a vertex polygon based on the the NEATLINE.
    'The calculation is based on the pixel and grid coordinates of the image corners.
    Private Function UnitRatio()
        'Declarations
        Dim DataLine() As String = Split(GdalContent, EndLine)          'Splitting GdalContent in to lines
        Dim LineCount As Integer = DataLine.Length - 1                  'Determening number of lines
        Dim Pointer As Integer
        Dim DeltaImage(4) As Double
        Dim DeltaMap(4) As Double
        Dim PreRatio(4) As Double
        Dim AvgRatio As Double

        'Clearing easting and northing of referencepoints
        For Pointer = 1 To 5
            ReferencePoint(Pointer).east = ""
            ReferencePoint(Pointer).north = ""
        Next

        'Retrieving/extracting width and Height of the image in grid coordinates.
        For Counter = 0 To LineCount
            '- Retrieving and easting and northing of the upper left corner
            If InStr(DataLine(Counter), "Upper Left", CompareMethod.Binary) > 0 Then
                Dim TmpString1() As String = Split(DataLine(Counter), ") (", CompareMethod.Binary)
                Dim TmpString2() As String = Split(TmpString1(0), "(", CompareMethod.Binary)
                Dim TmpString3() As String = Split(TmpString2(1), ",", CompareMethod.Binary)
                ReferencePoint(1).east = Trim(TmpString3(0))
                ReferencePoint(1).north = Trim(TmpString3(1))
            End If

            '- Retrieving and easting and northing of the upper right corner
            If InStr(DataLine(Counter), "Upper Right", CompareMethod.Binary) > 0 Then
                Dim TmpString1() As String = Split(DataLine(Counter), ") (", CompareMethod.Binary)
                Dim TmpString2() As String = Split(TmpString1(0), "(", CompareMethod.Binary)
                Dim TmpString3() As String = Split(TmpString2(1), ",", CompareMethod.Binary)
                ReferencePoint(2).east = Trim(TmpString3(0))
                ReferencePoint(2).north = Trim(TmpString3(1))
            End If

            '- Retrieving and easting and northing of the lower right corner
            If InStr(DataLine(Counter), "Lower Right", CompareMethod.Binary) > 0 Then
                Dim TmpString1() As String = Split(DataLine(Counter), ") (", CompareMethod.Binary)
                Dim TmpString2() As String = Split(TmpString1(0), "(", CompareMethod.Binary)
                Dim TmpString3() As String = Split(TmpString2(1), ",", CompareMethod.Binary)
                ReferencePoint(3).east = Trim(TmpString3(0))
                ReferencePoint(3).north = Trim(TmpString3(1))
            End If

            '- Retrieving and easting and northing of the lower left corner
            If InStr(DataLine(Counter), "Lower Left", CompareMethod.Binary) > 0 Then
                Dim TmpString1() As String = Split(DataLine(Counter), ") (", CompareMethod.Binary)
                Dim TmpString2() As String = Split(TmpString1(0), "(", CompareMethod.Binary)
                Dim TmpString3() As String = Split(TmpString2(1), ",", CompareMethod.Binary)
                ReferencePoint(4).east = Trim(TmpString3(0))
                ReferencePoint(4).north = Trim(TmpString3(1))
            End If

        Next
        'Calculating Delta's
        DeltaMap(1) = ((Val(ReferencePoint(1).east) - Val(ReferencePoint(2).east)) ^ 2 + (Val(ReferencePoint(1).north) - Val(ReferencePoint(2).north)) ^ 2) ^ 0.5
        DeltaMap(2) = ((Val(ReferencePoint(2).east) - Val(ReferencePoint(3).east)) ^ 2 + (Val(ReferencePoint(2).north) - Val(ReferencePoint(3).north)) ^ 2) ^ 0.5
        DeltaMap(3) = ((Val(ReferencePoint(3).east) - Val(ReferencePoint(4).east)) ^ 2 + (Val(ReferencePoint(3).north) - Val(ReferencePoint(4).north)) ^ 2) ^ 0.5
        DeltaMap(4) = ((Val(ReferencePoint(4).east) - Val(ReferencePoint(1).east)) ^ 2 + (Val(ReferencePoint(4).north) - Val(ReferencePoint(1).north)) ^ 2) ^ 0.5
        DeltaMap(0) = ((Val(ReferencePoint(1).east) - Val(ReferencePoint(3).east)) ^ 2 + (Val(ReferencePoint(1).north) - Val(ReferencePoint(3).north)) ^ 2) ^ 0.5

        'Calculating Ratio Map/Image
        ImageDiagonal = (ImageHeight ^ 2 + ImageWidth ^ 2) ^ 0.5
        PreRatio(1) = DeltaMap(1) / ImageWidth
        PreRatio(2) = DeltaMap(2) / ImageHeight
        PreRatio(3) = DeltaMap(3) / ImageWidth
        PreRatio(4) = DeltaMap(4) / ImageHeight
        PreRatio(0) = DeltaMap(0) / ImageDiagonal   'special case: testing grid is square (PreRation must ~ AvgRatio)
        'Calculating average ratio
        AvgRatio = (PreRatio(1) + PreRatio(2) + PreRatio(3) + PreRatio(4)) / 4
        'Scanning the difference between PreRatio and AvgRatio > to much > No vertex polygon (AvgRatio will set to 0)
        If Math.Abs(PreRatio(1) - AvgRatio) > 0.005 Or
                Math.Abs(PreRatio(2) - AvgRatio) > 0.005 Or
                Math.Abs(PreRatio(3) - AvgRatio) > 0.005 Or
                Math.Abs(PreRatio(4) - AvgRatio) > 0.005 Or
                Math.Abs(PreRatio(0) - AvgRatio) > 0.005 Then
            AvgRatio = 0
        End If

        'Returning value of function
        Return AvgRatio

    End Function

    'Subroutine SplitNeatLine extracts the grid coordinates of the NEATLINE polygon
    Private Sub SplitNeatLine()
        'Declarations
        Dim Pointer As Integer

        'Retrieving grid coordinates of NEATLINE points. (Last NEATLINE point = First point and will not be used)
        Dim TempString1() As String = Split(NeatLine, "((")
        Dim TempString2() As String = Split(TempString1(1), ",")
        VertexAmount = TempString2.Length - 2         'last item will not be used = same as first item (-2 in stead of -1)
        ReDim VertexPoint(VertexAmount + 1)           'Redim is needed to accomodated more then four vertex points
        For Pointer = 0 To VertexAmount
            Dim TempString3() As String = Split(TempString2(Pointer), " ")
            VertexPoint(Pointer + 1).east = TempString3(0)
            VertexPoint(Pointer + 1).north = TempString3(1)
        Next

    End Sub

    'Subroutine CalculateXY converts grid coordinates of the NEATLINE in to pixel coordinates of the vertex polygon.
    'Based on the distance of grid coordinates to the topleft (point A) and the bottomleft(Point B) corners of the image.
    Private Sub CalculateXY()
        'Declarations
        Dim Pointer As Integer
        Dim DistanceA, DistanceB As Double                      'Distance in pixel of vertex point to pont A and point B
        Dim Aeast As Double = Val(ReferencePoint(1).east)       'Grid easting of point A
        Dim Anorth As Double = Val(ReferencePoint(1).north)     'Grid northing of point B
        Dim Beast As Double = Val(ReferencePoint(4).east)       'Grid easting of Point B
        Dim Bnorth As Double = Val(ReferencePoint(4).north)     'Grid northing of Point B
        Dim Veast, Vnorth, Vx, Vy As Double

        For Pointer = 1 To VertexPoint.Length - 1
            Veast = Val(VertexPoint(Pointer).east)
            Vnorth = Val(VertexPoint(Pointer).north)
            'Calculating distance to point A and point B in pixels
            DistanceA = (((Aeast - Veast) ^ 2 + (Anorth - Vnorth) ^ 2) ^ 0.5) / Ratio
            DistanceB = (((Beast - Veast) ^ 2 + (Bnorth - Vnorth) ^ 2) ^ 0.5) / Ratio
            'Calculating y pixel
            Vy = (((DistanceA ^ 2 - DistanceB ^ 2) / (2 * ImageHeight)) + (0.5 * ImageHeight))
            VertexPoint(Pointer).y = System.Math.Round(Vy, 0)
            'Calculating x pixel
            Vx = (DistanceA ^ 2 - Vy ^ 2) ^ 0.5
            VertexPoint(Pointer).x = System.Math.Round(Vx, 0)
        Next

    End Sub

End Class
